@@ -1,13 +1,12 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-from __future__ import unicode_literals
+
 from collections import defaultdict
 
 from trytond.model import ModelSQL, ModelView, Workflow, fields
 from trytond.pyson import Eval, If, Bool
 from trytond.pool import Pool
 from trytond.transaction import Transaction
-from trytond import backend
 
 from trytond.modules.product import price_digits
 
@@ -98,44 +97,49 @@ class Complaint(Workflow, ModelSQL, ModelView):
         cls._buttons.update({
                 'cancel': {
                     'invisible': ~Eval('state').in_(['draft', 'waiting']),
+                    'depends': ['state'],
                     },
                 'draft': {
                     'invisible': ~Eval('state').in_(
                         ['waiting', 'done', 'cancelled']),
                     'icon': If(Eval('state').in_(['done', 'cancelled']),
-                        'tryton-clear', 'tryton-go-previous'),
+                        'tryton-undo', 'tryton-back'),
+                    'depends': ['state'],
                     },
                 'wait': {
                     'invisible': ~Eval('state').in_(['draft']),
+                    'depends': ['state'],
                     },
                 'approve': {
                     'invisible': ~Eval('state').in_(['waiting']),
+                    'depends': ['state'],
                     },
                 'reject': {
                     'invisible': ~Eval('state').in_(['waiting']),
+                    'depends': ['state'],
                     },
                 'process': {
                     'invisible': ~Eval('state').in_(['approved']),
+                    'depends': ['state'],
                     },
                 })
 
         origin_domain = []
-        for model, domain in cls._origin_domains().iteritems():
+        for model, domain in cls._origin_domains().items():
             origin_domain = If(Eval('origin_model') == model,
                 domain, origin_domain)
         cls.origin.domain = [origin_domain]
 
         actions_domains = cls._actions_domains()
         actions_domain = [('action', 'in', actions_domains.pop(None))]
-        for model, actions in actions_domains.iteritems():
+        for model, actions in actions_domains.items():
             actions_domain = If(Eval('origin_model') == model,
                 [('action', 'in', actions)], actions_domain)
         cls.actions.domain = [actions_domain]
 
     @classmethod
     def __register__(cls, module_name):
-        TableHandler = backend.get('TableHandler')
-        table_h = TableHandler(cls, module_name)
+        table_h = cls.__table_handler__(module_name)
 
         # Migration from 3.8: rename reference into number
         if (table_h.column_exist('reference')
@@ -231,8 +235,9 @@ class Complaint(Workflow, ModelSQL, ModelView):
     def copy(cls, complaints, default=None):
         if default is None:
             default = {}
-        default = default.copy()
-        default['number'] = None
+        else:
+            default = default.copy()
+        default.setdefault('number', None)
         return super(Complaint, cls).copy(complaints, default=default)
 
     @classmethod
@@ -285,11 +290,11 @@ class Complaint(Workflow, ModelSQL, ModelView):
                 result = action.do()
                 results[result.__class__].append(result)
                 actions[result.__class__].append(action)
-        for kls, records in results.iteritems():
+        for kls, records in results.items():
             kls.save(records)
             for action, record in zip(actions[kls], records):
                 action.result = record
-        Action.save(sum(actions.values(), []))
+        Action.save(sum(list(actions.values()), []))
 
 
 class Action(ModelSQL, ModelView):
